package org.mindtask.efs.boxnet;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.filesystem.IFileInfo;
import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.core.filesystem.IFileSystem;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.believer.songyuanframework.openapi.storage.box.constant.BoxConstant;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.CopyResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.CreateFolderResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.DeleteResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.MoveResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.impl.simple.objects.BoxAbstractFileImpl;
import cn.com.believer.songyuanframework.openapi.storage.box.objects.BoxAbstractFile;
import cn.com.believer.songyuanframework.openapi.storage.box.objects.BoxFolder;

/**
 * The box.net file store implementation
 * 
 * @author tanxu
 * @date Jan 21, 2012
 * @since 1.0
 */
public class BoxFileStore implements IFileStore {

	private static final Logger logger = LoggerFactory.getLogger(BoxFileStore.class);

	private BoxAbstractFile boxFile;
	private BoxFileInfo fileInfo;
	private UserInfo userInfo;
	private BoxFileSystem bfs;

	public BoxFileStore(BoxAbstractFile boxFile, UserInfo userInfo, BoxFileSystem boxFileSystem) {
		this.boxFile = boxFile;
		this.userInfo = userInfo;
		this.bfs = boxFileSystem;

		fileInfo = new BoxFileInfo(boxFile);
	}

	@Override
	public Object getAdapter(Class adapter) {
		if (adapter == IFileStore.class)
			return this;
		if (adapter == BoxAbstractFile.class)
			return boxFile;
		else if (adapter == IFileSystem.class)
			return bfs;
		else if (adapter == IFileInfo.class)
			return fileInfo;
		return null;
	}

	@Override
	public IFileInfo[] childInfos(int options, IProgressMonitor monitor) throws CoreException {
		IFileStore[] childStores = childStores(options, monitor);
		IFileInfo[] infos = new IFileInfo[childStores.length];
		int i = 0;
		for (IFileStore store : childStores) {
			infos[i++] = store.fetchInfo();
		}
		return infos;
	}

	@Override
	public String[] childNames(int options, IProgressMonitor monitor) throws CoreException {
		IFileStore[] childStores = childStores(options, monitor);
		String[] names = new String[childStores.length];
		int i = 0;
		for (IFileStore store : childStores) {
			names[i++] = store.getName();
		}
		return names;
	}

	@Override
	public IFileStore[] childStores(int options, IProgressMonitor monitor) throws CoreException {
		if (boxFile.isFolder()) {
			List<BoxAbstractFile> childBoxFiles;
			try {
				childBoxFiles = bfs.getChildren(this);
			}
			catch (Exception e) {
				throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to get children for [" + getName()
						+ "]: " + e.getMessage(), e));
			}
			IFileStore[] childStores = new IFileStore[childBoxFiles.size()];
			int i = 0;
			for (BoxAbstractFile file : childBoxFiles) {
				childStores[i++] = new BoxFileStore(file, userInfo, bfs);
			}
			return childStores;
		}
		return new IFileStore[0];
	}

	@Override
	public void copy(IFileStore destination, int options, IProgressMonitor monitor) throws CoreException {
		if (fetchInfo().isDirectory())
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "Copy folder [" + getName()
					+ "] is not supported!"));
		CopyResponse copyResponse = null;
		try {
			copyResponse = bfs.copy(this, (BoxFileStore) destination);
		}
		catch (Exception e) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to copy [" + this.getName() + "] to ["
					+ destination.getName() + "]: " + e.getMessage(), e));
		}
		if (!BoxConstant.STATUS_S_COPY_NODE.equals(copyResponse.getStatus())) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to copy [" + this.getName() + "] to ["
					+ destination.getName() + "]: " + copyResponse.getStatus()));
		}
	}

	@Override
	public void delete(int options, IProgressMonitor monitor) throws CoreException {
		DeleteResponse deleteResponse = null;
		try {
			deleteResponse = bfs.delete(this);
		}
		catch (Exception e) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to delete file [" + boxFile.getName()
					+ "]: " + e.getMessage(), e));
		}
		if (!BoxConstant.STATUS_S_DELETE_NODE.equals(deleteResponse.getStatus())) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to delete file [" + boxFile.getName()
					+ "]: " + deleteResponse.getStatus()));
		}
	}

	@Override
	public BoxFileInfo fetchInfo() {
		return fileInfo;
	}

	@Override
	public BoxFileInfo fetchInfo(int options, IProgressMonitor monitor) throws CoreException {
		try {
			if (boxFile.isFolder()) {
				boxFile = bfs.getFolderById(boxFile.getId(), userInfo.getAuthToken());
			}
			else {
				boxFile = bfs.getFileById(boxFile.getId(), userInfo.getAuthToken());
			}
		}
		catch (Exception e) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, e.getMessage(), e));
		}

		fileInfo = new BoxFileInfo(boxFile);
		return fileInfo;
	}

	@Override
	public BoxFileStore getChild(IPath path) {
		return getFileStore(path);
	}

	@Override
	public BoxFileStore getFileStore(IPath path) {
		// alway return one file store, but set the exists as false if it does not exist
		BoxFileStore result = this;
		for (int i = 0; i < path.segmentCount(); i++) {
			String name = path.segment(i);
			if ("..".equals(name))
				result = result.getParent();
			else if (!".".equals(name))
				result = result.getChild(name);
		}
		return result;
	}

	@Override
	public BoxFileStore getChild(String name) {
		BoxFileStore fileStore = null;
		BoxAbstractFile childFile = null;
		try {
			childFile = bfs.getChild(this, name);
		}
		catch (Exception e) {
			logger.warn("get child file, create handler for it", e);
		}
		if (childFile == null) {
			childFile = new BoxAbstractFileImpl();
			childFile.setName(name);
			fileStore = new BoxFileStore(childFile, userInfo, bfs);
			BoxFileInfo info = fileStore.fetchInfo();
			info.setExists(false);
		}
		else {
			fileStore = new BoxFileStore(childFile, userInfo, bfs);
		}

		// need consider how to deal with the parent id in consistent way
		childFile.setParentId(this.boxFile.getId());
		return fileStore;
	}

	@Override
	public BoxFileSystem getFileSystem() {
		return bfs;
	}

	@Override
	public String getName() {
		return boxFile.getName();
	}

	@Override
	public BoxFileStore getParent() {
		if (!isRoot()) {
			if ("0".equals(boxFile.getParentId())) {
				return getRootFile(userInfo, bfs);
			}
			else {
				try {
					BoxAbstractFile file = bfs.getFolderById(boxFile.getParentId(), getUserInfo().getAuthToken());
					return new BoxFileStore(file, getUserInfo(), bfs);
				}
				catch (Exception e) {
					logger.error("get parent folder failed: " + e.getMessage(), e);
				}
			}
		}
		return null;
	}

	@Override
	public boolean isParentOf(IFileStore other) {
		if (!fetchInfo().isDirectory())
			return false;
		if (!(other instanceof BoxFileStore))
			return false;

		BoxAbstractFile otherBoxFile = ((BoxFileStore) other).getBoxFile();
		try {
			List<BoxAbstractFile> boxFiles = bfs.getChildren(this);
			for (BoxAbstractFile bfile : boxFiles) {
				if (bfile.getId().equals(otherBoxFile.getId())) {
					return true;
				}
			}
		}
		catch (Exception e) {
			logger.error("Failed to get children list of folder [{}]: {}", getName(), e);
		}

		return false;
	}

	@Override
	public IFileStore mkdir(int options, IProgressMonitor monitor) throws CoreException {
		if (fetchInfo().exists())
			return this;
		CreateFolderResponse createFolderResponse = null;
		try {
			createFolderResponse = bfs.createFolder(this);
		}
		catch (Exception e) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to create folder [" + getName() + "]: "
					+ e.getMessage(), e));
		}

		if (!BoxConstant.STATUS_CREATE_OK.equals(createFolderResponse.getStatus())
				&& !BoxConstant.STATUS_S_FOLDER_EXISTS.equals(createFolderResponse.getStatus())) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to create folder [" + getName() + "]:"
					+ createFolderResponse.getStatus()));
		}
		BoxFolder boxFolder = createFolderResponse.getFolder();
		return new BoxFileStore(boxFolder, userInfo, bfs);
	}

	@Override
	public void move(IFileStore destination, int options, IProgressMonitor monitor) throws CoreException {
		MoveResponse moveRespone = null;
		try {
			moveRespone = bfs.move(this, (BoxFileStore) destination);
		}
		catch (Exception e) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to move [" + this.getName() + "] to ["
					+ destination.getName() + "]: " + e.getMessage(), e));
		}
		if (!BoxConstant.STATUS_S_MOVE_NODE.equals(moveRespone.getStatus())) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to move [" + this.getName() + "] to ["
					+ destination.getName() + "]: " + moveRespone.getStatus()));
		}
	}

	@Override
	public InputStream openInputStream(int options, IProgressMonitor monitor) throws CoreException {
		if (!fetchInfo().isDirectory()) {
			try {
				byte[] byRawData = bfs.downloadFile(this);
				return new ByteArrayInputStream(byRawData);
			}
			catch (Exception e) {
				throw new CoreException(Activator.newStatus(IStatus.ERROR, "Failed to download file [" + getName()
						+ "]: " + e.getMessage(), e));
			}
		}
		return null;
	}

	@Override
	public OutputStream openOutputStream(int options, IProgressMonitor monitor) throws CoreException {
		if (boxFile.isFolder())
			throw new CoreException(Activator.newStatus(IStatus.ERROR, "This store represents a directory"));
		return new BoxOutputStream(this);
	}

	@Override
	public void putInfo(IFileInfo info, int options, IProgressMonitor monitor) throws CoreException {
		throw new RuntimeException("Not yet implemented");
	}

	@Override
	public File toLocalFile(int options, IProgressMonitor monitor) throws CoreException {
		try {
			return File.createTempFile("boxnet", "mapping");
		}
		catch (IOException e) {
			throw new CoreException(Activator.newStatus(IStatus.ERROR, e.getMessage(), e));
		}
	}

	@Override
	public URI toURI() {
		StringBuilder builder = new StringBuilder();
		builder.append("boxnet://");
		builder.append(getUserInfo().getName());
		builder.append(':');
		builder.append(getUserInfo().getAuthToken());
		builder.append("@box.net");
		ArrayList<BoxFileStore> parents = new ArrayList<BoxFileStore>();
		BoxFileStore fileStore = this;
		while (fileStore != null) {
			parents.add(fileStore);
			fileStore = fileStore.getParent();
		}
		for (int i = parents.size() - 1; i >= 0; i--) {
			builder.append(parents.get(i).getName());
			builder.append('/');
		}
		if (builder.charAt(builder.length() - 1) == '/') {
			builder.deleteCharAt(builder.length() - 1);
		}
		try {
			return new URI(builder.toString());
		}
		catch (URISyntaxException e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}

	public static BoxFileStore getRootFile(UserInfo userInfo, BoxFileSystem boxFileSystem) {
		BoxAbstractFileImpl rootFile = new BoxAbstractFileImpl();
		rootFile.setId("0");
		rootFile.setFolder(true);
		rootFile.setName("");

		return new BoxFileStore(rootFile, userInfo, boxFileSystem);
	}

	public boolean isRoot() {
		return (boxFile.getParentId() == null || "0".equals(boxFile.getId()));
	}

	protected BoxAbstractFile getBoxFile() {
		return this.boxFile;
	}

	protected UserInfo getUserInfo() {
		return this.userInfo;
	}

	protected void setBoxFile(BoxAbstractFile boxFile) {
		this.boxFile = boxFile;
	}
}
