package org.mindtask.efs.boxnet;

import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.eclipse.core.filesystem.EFS;
import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.core.filesystem.provider.FileSystem;
import org.eclipse.core.runtime.Path;

import cn.com.believer.songyuanframework.openapi.storage.box.BoxExternalAPI;
import cn.com.believer.songyuanframework.openapi.storage.box.constant.BoxConstant;
import cn.com.believer.songyuanframework.openapi.storage.box.factories.BoxRequestFactory;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.CopyRequest;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.CopyResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.CreateFolderRequest;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.CreateFolderResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.DeleteRequest;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.DeleteResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.DownloadRequest;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.DownloadResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.GetAccountTreeRequest;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.GetAccountTreeResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.GetFileInfoRequest;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.GetFileInfoResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.MoveRequest;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.MoveResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.UploadRequest;
import cn.com.believer.songyuanframework.openapi.storage.box.functions.UploadResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.impl.simple.SimpleBoxImpl;
import cn.com.believer.songyuanframework.openapi.storage.box.objects.BoxAbstractFile;
import cn.com.believer.songyuanframework.openapi.storage.box.objects.BoxException;
import cn.com.believer.songyuanframework.openapi.storage.box.objects.UploadResult;

import com.google.common.base.Strings;

/**
 * The box.net file system provider.
 * <p>
 * <code>IFileSystem</code> instances in EFS is kinda singleton if calling {@link EFS#getFileSystem(String)}, according
 * to the implementation of org.eclipse.core.internal.filesystem.InternalFileSystemCore.getFileSystem(String); <br/>
 * need consider how to manage the authentication next.
 * </p>
 * 
 * @author tanxu
 * @date Jan 22, 2012
 * @since 1.0
 */
public class BoxFileSystem extends FileSystem {

	static final String apiKey = "xnt44lj8xokcxtxndyxqdcd8ss9hdv5i";
	private BoxExternalAPI boxExternalAPI;

	/**
	 * Scheme constant (value "boxnet") indicating the Box.net file system scheme.
	 */
	public static final String SCHEME_BOXNET = "boxnet"; //$NON-NLS-1$

	public BoxFileSystem() {
		boxExternalAPI = new SimpleBoxImpl();
	}

	@Override
	public IFileStore getStore(URI uri) {
		if (SCHEME_BOXNET.equals(uri.getScheme())) {
			String user = uri.getUserInfo();
			if (!Strings.isNullOrEmpty(user)) {
				UserInfo userInfo = null;
				int idx = user.indexOf(':');
				if (idx > 0) {
					// now just simplify the user authentication, that put the auth_token into the URI
					// TODO to support oauth like box.net authentication
					userInfo = new UserInfo(user.substring(0, idx), user.substring(idx + 1));
				}
				BoxFileStore rootFileStore = BoxFileStore.getRootFile(userInfo, this);
				String path = uri.getPath();
				if (path.isEmpty() || "/".equals(path)) {
					return rootFileStore;
				}
				else {
					return rootFileStore.getChild(new Path(path));
				}
			}
		}
		return EFS.getNullFileSystem().getStore(uri);
	}

	/**
	 * @param fileStore the parent file store
	 * @param childName the child file name
	 * @return the child box file, or <code>null</code> if not found
	 * @throws BoxException
	 * @throws IOException
	 */
	public BoxAbstractFile getChild(BoxFileStore fileStore, String childName) throws IOException, BoxException {
		GetAccountTreeRequest getAccountTreeRequest = BoxRequestFactory.createGetAccountTreeRequest(apiKey, fileStore
				.getUserInfo().getAuthToken(), fileStore.getBoxFile().getId(), new String[] {
				"nozip",
				"onelevel"
		});
		GetAccountTreeResponse getAccountTreeResponse = boxExternalAPI.getAccountTree(getAccountTreeRequest);
		if (!BoxConstant.STATUS_LISTING_OK.equals(getAccountTreeResponse.getStatus())) {
			throw new BoxException("Failed to retrieve children for file [" + fileStore.getName() + "]: "
					+ getAccountTreeResponse.getStatus());
		}
		Enumeration<DefaultMutableTreeNode> en = getAccountTreeResponse.getTree().children();
		while (en.hasMoreElements()) {
			DefaultMutableTreeNode node = en.nextElement();
			BoxAbstractFile boxFile = (BoxAbstractFile) node.getUserObject();
			if (boxFile.getName().equals(childName))
				return boxFile;
		}

		return null;
	}

	public BoxAbstractFile getFolderById(final String folderId, String authToken) throws IOException, BoxException {
		GetAccountTreeRequest getAccountTreeRequest = BoxRequestFactory.createGetAccountTreeRequest(apiKey, authToken,
				"0", new String[] {
						"nozip",
						"nofiles"
				});
		GetAccountTreeResponse getAccountTreeResponse = boxExternalAPI.getAccountTree(getAccountTreeRequest);
		if (!BoxConstant.STATUS_LISTING_OK.equals(getAccountTreeResponse.getStatus())) {
			throw new BoxException("Failed to retrieve children for folder [" + folderId + "]: "
					+ getAccountTreeResponse.getStatus());
		}

		FolderVisitor collector = new FolderVisitor(folderId);
		visitTree(getAccountTreeResponse.getTree(), collector);
		return collector.getFile();
	}

	public BoxAbstractFile getFileById(String fileId, String authToken) throws IOException, BoxException {
		GetFileInfoRequest getFileInfoRequest = BoxRequestFactory.createGetFileInfoRequest(apiKey, authToken, fileId);
		GetFileInfoResponse getFileInfoResponse = boxExternalAPI.getFileInfo(getFileInfoRequest);
		if (!BoxConstant.STATUS_S_GET_FILE_INFO.equals(getFileInfoResponse.getStatus())) {
			throw new BoxException("Failed to get file info for file [" + fileId + "]: "
					+ getFileInfoResponse.getStatus());
		}

		return getFileInfoResponse.getFile();
	}

	private void visitTree(DefaultMutableTreeNode node, ResourceVisitor collector) {
		Enumeration<DefaultMutableTreeNode> en = node.children();
		while (en.hasMoreElements()) {
			DefaultMutableTreeNode n = en.nextElement();
			if (collector.visit(n)) {
				visitTree(n, collector);
			}
		}
	}

	/**
	 * @param fileStore
	 * @throws BoxException
	 * @throws IOException
	 */
	public DeleteResponse delete(BoxFileStore fileStore) throws IOException, BoxException {
		String target = fileStore.fetchInfo().isDirectory() ? "folder" : "file";
		DeleteRequest deleteRequest = BoxRequestFactory.createDeleteRequest(apiKey, fileStore.getUserInfo()
				.getAuthToken(), target, fileStore.getBoxFile().getId());
		return boxExternalAPI.delete(deleteRequest);
	}

	/**
	 * @param boxFileStore
	 * @return
	 * @throws BoxException
	 * @throws IOException
	 */
	public CreateFolderResponse createFolder(BoxFileStore fileStore) throws IOException, BoxException {
		BoxFileStore parentFileStore = (BoxFileStore) fileStore.getParent();
		String parentFolderId = "0"; // create folder under root by default
		if (parentFileStore != null) {
			parentFileStore.getBoxFile().getId();
		}
		CreateFolderRequest createFolderRequest = BoxRequestFactory.createCreateFolderRequest(apiKey, fileStore
				.getUserInfo().getAuthToken(), parentFolderId, fileStore.getName(), false);
		CreateFolderResponse createFolderResponse = boxExternalAPI.createFolder(createFolderRequest);
		return createFolderResponse;
	}

	/**
	 * @param fileStore the parent file store
	 * @return list of <code>BoxAbstractFile</code>
	 * @throws BoxException
	 * @throws IOException
	 */
	public List<BoxAbstractFile> getChildren(BoxFileStore fileStore) throws IOException, BoxException {
		GetAccountTreeRequest getAccountTreeRequest = BoxRequestFactory.createGetAccountTreeRequest(apiKey, fileStore
				.getUserInfo().getAuthToken(), fileStore.getBoxFile().getId(), new String[] {
				"nozip",
				"onelevel"
		});
		GetAccountTreeResponse getAccountTreeResponse = boxExternalAPI.getAccountTree(getAccountTreeRequest);
		if (!BoxConstant.STATUS_LISTING_OK.equals(getAccountTreeResponse.getStatus())) {
			throw new BoxException("Failed to retrieve children for file [" + fileStore.getName() + "]: "
					+ getAccountTreeResponse.getStatus());
		}

		List<BoxAbstractFile> files = new ArrayList<BoxAbstractFile>();
		for (Enumeration en = getAccountTreeResponse.getTree().children(); en.hasMoreElements();) {
			files.add((BoxAbstractFile) ((DefaultMutableTreeNode) en.nextElement()).getUserObject());
		}
		return files;
	}

	/**
	 * @param boxFileStore
	 * @return
	 * @throws BoxException
	 * @throws IOException
	 */
	public byte[] downloadFile(BoxFileStore fileStore) throws IOException, BoxException {
		DownloadRequest downloadRequest = BoxRequestFactory.createDownloadRequest(fileStore.getUserInfo()
				.getAuthToken(), fileStore.getBoxFile().getId(), false, null);
		DownloadResponse downloadRepsone = boxExternalAPI.download(downloadRequest);
		return downloadRepsone.getRawData();
	}

	public UploadResponse uploadFile(BoxFileStore fileStore, byte[] content) throws IOException, BoxException {
		BoxFileStore parentFileStore = (BoxFileStore) fileStore.getParent();
		String parentFileId = "0";
		if (parentFileStore != null) {
			parentFileId = parentFileStore.getBoxFile().getId();
		}
		Map nameValueMap = new HashMap(1);
		nameValueMap.put(fileStore.getName(), content);
		UploadRequest uploadRequest = BoxRequestFactory.createUploadRequest(fileStore.getUserInfo().getAuthToken(),
				false, parentFileId, nameValueMap);
		UploadResponse uploadResponse = boxExternalAPI.upload(uploadRequest);
		// update parent id
		List uploadResultList = uploadResponse.getUploadResultList();
		for (int i = 0; i < uploadResultList.size(); i++) {
			UploadResult uploadResult = (UploadResult) uploadResultList.get(i);
			uploadResult.getFile().setParentId(parentFileId);
		}

		return uploadResponse;
	}

	public interface ResourceVisitor {
		boolean visit(DefaultMutableTreeNode node);
	}

	private static class FolderVisitor implements ResourceVisitor {

		private BoxAbstractFile file;
		public String folderId;

		public FolderVisitor(String folderId) {
			this.folderId = folderId;
		}

		@Override
		public boolean visit(DefaultMutableTreeNode node) {
			BoxAbstractFile boxFile = ((BoxAbstractFile) node.getUserObject());
			if (boxFile.isFolder() && boxFile.getId().equals(folderId)) {
				file = boxFile;
				return false;
			}
			return true;
		}

		public BoxAbstractFile getFile() {
			return this.file;
		}
	}

	/**
	 * @param boxFileStore
	 * @param destination
	 * @return
	 * @throws BoxException
	 * @throws IOException
	 */
	public MoveResponse move(BoxFileStore sourceStore, BoxFileStore destination) throws IOException, BoxException {
		String target = sourceStore.fetchInfo().isDirectory() ? "folder" : "file";
		MoveRequest moveRequest = BoxRequestFactory.createMoveRequest(apiKey, sourceStore.getUserInfo().getAuthToken(),
				target, sourceStore.getBoxFile().getId(), destination.getBoxFile().getId());
		return boxExternalAPI.move(moveRequest);
	}

	/**
	 * @param boxFileStore
	 * @param destination
	 * @return
	 * @throws BoxException
	 * @throws IOException
	 */
	public CopyResponse copy(BoxFileStore sourceStore, BoxFileStore destination) throws IOException, BoxException {
		String target = sourceStore.fetchInfo().isDirectory() ? "folder" : "file";
		CopyRequest copyRequest = BoxRequestFactory.createCopyRequest(apiKey, sourceStore.getUserInfo().getAuthToken(),
				target, sourceStore.getBoxFile().getId(), destination.getBoxFile().getId());
		return boxExternalAPI.copy(copyRequest);
	}
}
